@@ -1,16 +1,12 @@
+"""util for basic statistical steps"""
+
+from typing import Tuple
+
 import numpy
 
 import scipy.stats
 import scipy.optimize
-
-have_sklearn = False
-# noinspection PyBroadException
-try:
-    import sklearn.linear_model
-
-    have_sklearn = True
-except Exception:
-    pass
+import sklearn.linear_model
 
 
 # methods to avoid calling statsmodels which seems to be incompatible with many
@@ -18,8 +14,15 @@ except Exception:
 #  https://github.com/WinVector/pyvtreat/issues/14
 
 
-def our_corr_score(*, y_true, y_pred):
-    # compute Pearson correlation
+def our_corr_score(*, y_true, y_pred) -> Tuple[float, float]:
+    """
+    Compute Pearson correlation. Case-out some corner cases.
+
+    :param y_true: truth values
+    :param y_pred: predictions
+    :return: (pearson r, significance)
+    """
+
     if not isinstance(y_true, numpy.ndarray):
         y_true = numpy.asarray(y_true)
     if not isinstance(y_pred, numpy.ndarray):
@@ -37,54 +40,44 @@ def our_corr_score(*, y_true, y_pred):
     return r, sig
 
 
-def est_deviance(*, y, est, epsilon=1.0e-5):
+def est_deviance(*, y, est, epsilon: float = 1.0e-5) -> float:
+    """
+    Estimate the deviance
+
+    :param y: truth values
+    :param est: predictions
+    :param epsilon: how close to get to 0 and 1
+    :return: deviance estimate
+    """
+
     if not isinstance(y, numpy.ndarray):
         y = numpy.asarray(y)
     if not isinstance(est, numpy.ndarray):
-        x = numpy.asarray(est)
+        est = numpy.asarray(est)
     est = numpy.minimum(est, 1 - epsilon)
     est = numpy.maximum(est, epsilon)
-    deviance = -2 * numpy.sum(
-        y * numpy.log(est) +
-        (1 - y) * numpy.log(1 - est))
+    deviance = -2 * numpy.sum(y * numpy.log(est) + (1 - y) * numpy.log(1 - est))
     return deviance
 
 
-# assumes special cases of solve_logistic_regression already eliminated
-def brute_force_solve_logistic(*, y, x, regularization=1.e-6):
-    if not isinstance(y, numpy.ndarray):
-        y = numpy.asarray(y)
-    if not isinstance(x, numpy.ndarray):
-        x = numpy.asarray(x)
+def sklearn_solve_logistic(*, y, x, regularization: float = 1.0e-6):
+    """
+    Single variable logistic regression.
+    Assumes special cases of solve_logistic_regression already eliminated.
 
-    def predict(beta):
-        return 1 / (1 + numpy.exp(-(beta[0] + beta[1] * x)))
+    :param y: dependent variable
+    :param x: explanatory variable
+    :param regularization:
+    :return: model predictions
+    """
 
-    def loss(beta):
-        preds = predict(beta)
-        return (est_deviance(y=y, est=preds)
-                + regularization * (beta[0] * beta[0] + beta[1] * beta[1])  # regularization
-                )
-
-    soln = scipy.optimize.fmin_bfgs(
-        loss,
-        x0=[numpy.log(numpy.mean(y) / (1 - numpy.mean(y))), 0.001],
-        gtol=1.0e-3,
-        disp=0)
-    return predict(soln)
-
-
-# assumes special cases of solve_logistic_regression already eliminated
-def sklearn_solve_logistic(*, y, x, regularization=1.e-6):
     if not isinstance(y, numpy.ndarray):
         y = numpy.asarray(y)
     if not isinstance(x, numpy.ndarray):
         x = numpy.asarray(x)
     fitter = sklearn.linear_model.LogisticRegression(
-        penalty='l2',
-        solver='lbfgs',
-        fit_intercept=True,
-        C=1/regularization)
+        penalty="l2", solver="lbfgs", fit_intercept=True, C=1 / regularization
+    )
     dependent_vars = x.reshape((len(y), 1))
     fitter.fit(X=dependent_vars, y=y)
     preds = fitter.predict_proba(X=dependent_vars)[:, 1]
@@ -93,17 +86,26 @@ def sklearn_solve_logistic(*, y, x, regularization=1.e-6):
 
 # x, y - numpy numeric vectors, y 0/1.  solve for y- return predictions
 def solve_logistic_regression(*, y, x):
-    # catch some corner cases
+    """
+    Single variable logistic regression. Returns predictions, corner
+    cases removed.
+
+    :param y: dependent variable
+    :param x: explanatory variable
+    :return: predictions
+    """
+
     if not isinstance(y, numpy.ndarray):
         y = numpy.asarray(y)
     if not isinstance(x, numpy.ndarray):
         x = numpy.asarray(x)
+    # catch some corner cases
     n = len(y)
     if (n < 2) or (numpy.min(y) >= numpy.max(y)):
         return y.copy()
     if numpy.min(x) >= numpy.max(x):
         return numpy.asarray([numpy.mean(y)] * n)
-    # check for fully seperable cases
+    # check for fully separable cases
     big_y_indices = y > 0
     x_b = x[big_y_indices]
     x_s = x[numpy.logical_not(big_y_indices)]
@@ -112,15 +114,20 @@ def solve_logistic_regression(*, y, x):
         r[big_y_indices] = 1
         return r
     # run a full logistic regression
-    if have_sklearn:
-        preds = sklearn_solve_logistic(y=y, x=x)
-    else:
-        preds = brute_force_solve_logistic(y=y, x=x)
+    preds = sklearn_solve_logistic(y=y, x=x)
     return numpy.asarray(preds)
 
 
 # noinspection PyPep8Naming
-def our_pseudo_R2(*, y_true, y_pred):
+def our_pseudo_R2(*, y_true, y_pred) -> Tuple[float, float]:
+    """
+    Return the logistic pseudo-R2
+
+    :param y_true: dependent variable
+    :param y_pred: explanatory variable
+    :return: (pseudo-R2, significance)
+    """
+
     if not isinstance(y_true, numpy.ndarray):
         y_true = numpy.asarray(y_true)
     if not isinstance(y_pred, numpy.ndarray):
